diff --git a/res/res_pjsip_registrar.c b/res/res_pjsip_registrar.c
index c16a36fe59..7db2107f22 100644
--- a/res/res_pjsip_registrar.c
+++ b/res/res_pjsip_registrar.c
@@ -313,6 +313,7 @@ static int registrar_validate_path(pjsip_rx_data *rdata, struct ast_sip_aor *aor
 
 /*! Transport monitor for incoming REGISTER contacts */
 struct contact_transport_monitor {
+	int removing;
 	/*!
 	 * \brief Sorcery contact name to remove on transport shutdown
 	 * \note Stored after aor_name in space reserved when struct allocated.
@@ -344,6 +345,16 @@ static int register_contact_transport_remove_cb(void *data)
 	}
 
 	ao2_lock(lock);
+
+	if (monitor->removing) {
+		ao2_unlock(lock);
+		ast_named_lock_put(lock);
+		ao2_ref(monitor, -1);
+		return 0;
+	}
+
+	monitor->removing = 1;
+
 	contact = ast_sip_location_retrieve_contact(monitor->contact_name);
 	if (contact) {
 		ast_sip_location_delete_contact(contact);
@@ -358,6 +369,7 @@ static int register_contact_transport_remove_cb(void *data)
 			contact->user_agent);
 		ao2_ref(contact, -1);
 	}
+
 	ao2_unlock(lock);
 	ast_named_lock_put(lock);
 
@@ -379,6 +391,10 @@ static void register_contact_transport_shutdown_cb(void *data)
 {
 	struct contact_transport_monitor *monitor = data;
 
+	if (monitor->removing) {
+		return;
+	}
+
 	/*
 	 * Push off to a default serializer.  This is in case sorcery
 	 * does database accesses for contacts.  Database accesses may
